/** 
 *   blob: lectura escritura de los bloques <BLOB>..</BLOB> fuera del
 *   header xml
 *
 *   (c) Copyright 2010 Federico Bareilles <bareilles@gmail.com>,
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *   
 **/

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include <stdint.h>
//#include <unistd.h>
#include <my_stdio.h>
#include <malloc.h>
#include <line_conf.h>

#include "blob.h"
#include "my_xcomp.h"


static char *comp_srt[]={"none","qt"};


static char * stripchar (char *string, char c)
{
        char *s = string, *t;

	if(*s == c) s++;
	t = s + strlen (s) - 1;
	if(*t == c) *t = '\0';
	
        return s;
}


static int parse_blob_line(char *line, struct blob *bl)
{
	char *sline = stripwhite(line);
	char *saveptr = NULL;
	int ret = 0;
	char *tag, *data, *dummy;

	if ( strncmp(sline, "<BLOB ", 6) == 0 ) {
		strtok_r(sline, " >", &saveptr);
		while(  sline != NULL) {
			parse_line(sline, &tag, &data, &dummy);
			if (tag != NULL && data != NULL) {
				data = stripchar(data, '"');
				if (strncmp(tag, "blobid", 6) == 0 ) {
					bl->id = atoi(data);
				} else if (strncmp(tag, "size", 4) == 0 ) {
					bl->size = atoi(data);
				} else if (strncmp(tag, 
						   "compression", 11) == 0 ) {
					if (strncmp(data, "qt", 2) == 0 )
						bl->compress_flag = 1;
					else
						bl->compress_flag = 0;
				}
				
			}
			sline = strtok_r(NULL, " >", &saveptr);
		}
		ret = 1; 
	}

	return ret;
}


static int if_end_blob(char *line) 
{
	int ret = 0;

	if ( strncmp(line, "</BLOB>", 7) == 0 )
		ret = 1;

	return ret;
}



static int blob_get(FILE *fp, struct blob *bl)
{
	int done = 0;
	int ret = -2;
	char * line = NULL;
	ssize_t read;
	size_t len = 0;

	bl->size = -1;
	bl->data = NULL;
	bl->parse_header_flag = 0;
	/* get next blob header in data area: */
	do {
		read = getline(&line, &len, fp);
		if ( read != -1)
			done = parse_blob_line(line, bl);
	} while (!done && read != -1);
	if (done && bl->size > 0) {
		bl->data = (unsigned char *) malloc(
			sizeof(unsigned char) * bl->size);
		read = fread(bl->data, sizeof(unsigned char), bl->size, fp);
		if(read == bl->size) {
			read = getline(&line, &len, fp); /* read last 0xa */
			if(read == 1 && *line == 0x0a) {
				read = getline(&line, &len, fp);
				if (if_end_blob(line)) 
					ret = 0;
			}
			}
	} else if (read <0) {
		PVERB(PINFO,"EOF");
		ret = -1;
	}
	if (line)
		free(line);
	if (ret != 0) {
		if (ret != -1)
			PVERB(PERR, "Error reading BLOB data area.");
		if (bl->data != NULL) {
			free (bl->data);
			bl->data = NULL;
		}
	}
	
	return ret;
}


static struct blob *make_blob(void)
{
	struct blob *bl = (struct blob *) malloc(sizeof(struct blob));

	if( bl != NULL) {
		BLOB_GET_ID(bl) = -1;
		bl->size = -1;
		BLOB_GET_BLOB_TYPE(bl) = BT_UNKNOW;
		BLOB_GET_DATA(bl) = NULL;
		bl->parse_header_flag = 0;
		bl->next = NULL;
	}

	return bl;
}


static void free_blob(struct blob *bl)
{
	if(bl->data != NULL) free(bl->data);
	free(bl);
	bl = NULL;

	return;
}


int blob_get_file(const char *fname, struct blob **abl)
{
	FILE * fp;
	struct blob *bl = make_blob();
	struct blob *blp, *bllast;
	int ret;
	int count = 0;

	if ((fp = fopen(fname, "r"))  == NULL) {
		PVERB(PERR,"Can't open file: %s", fname);
		return -1;
	}

	bllast = blp = bl;
	do {
		if ((ret = blob_get(fp, blp)) == 0) {
			count++;
			blp->next = make_blob();
			bllast = blp;
			blp = blp->next;
		}
	} while(ret == 0);
	fclose(fp);

	if(blp == bl) { /* nothing */
		free_blob(bl);
		bl = NULL;
	} else if (blp->size < 0) {
		free_blob(blp);
		blp = NULL;
		if (bllast != NULL)
			bllast->next = NULL;
	}
	*abl = bl;

	if (bl == NULL) {
		return -1;
	}
	return count;
}


int blob_uncompress(struct blob *bl)
{
	int ret = 0;
	unsigned char *data_out = NULL;
	size_t size_out;

	if (bl->compress_flag != 1) {
		ret = -1;
	} else {
		data_out = (unsigned char *) 
			my_uncomp((void *) bl->data, bl->size, &size_out);
		if (data_out != NULL) {
			free(bl->data);
			bl->data = data_out;
			bl->size = size_out;
			bl->compress_flag = 0;
		} else {
			ret = -2;
		}
	}

	return ret;
}


int blob_compress(struct blob *bl)
{
	int ret = 0;
	unsigned char *data_out = NULL;
	size_t size_out;

	if (bl->compress_flag != 0) {
		ret = -1;
	} else {
		data_out = (unsigned char *) 
			my_comp((void *) bl->data, bl->size, &size_out);
		if (data_out != NULL) {
			free(bl->data);
			bl->data = data_out;
			bl->size = size_out;
			bl->compress_flag = 1;
		} else {
			ret = -2;
		}
	}

	return ret;
}


void free_blob_all(struct blob *bl)
{
	struct blob *blp, *blf;
	
	blp = bl;
	while( blp != NULL) {
		blf = blp;
		blp = blp->next;
		free_blob(blf);
	}
	bl = NULL;

	return;
}


/*
<!-- END XML -->
<BLOB blobid="0" size="737" compression="qt">
*/
int blob_print(FILE *fp, struct blob *bl, int compress_flag)
{
	int count = 0;
	struct blob *blp;

	for(blp = bl; blp != NULL; blp = blp->next) {
		if( compress_flag == 1 && blp->compress_flag == 0)
			blob_compress(blp);
		else if(compress_flag == 0 && blp->compress_flag == 1)
			blob_uncompress(blp);
		count += fprintf(
			fp,
			"<BLOB blobid=\"%d\" size=\"%d\" compression=\"%s\">\n",
			blp->id, blp->size, comp_srt[blp->compress_flag]);
		count += fwrite(blp->data, sizeof(char), blp->size, fp);
		/* NOTE: el archivo original termina con 0xd 0xa, por
		 * ahora termino con 0xa solamente. */
		count += fprintf(fp, "\n</BLOB>\n");
	}

	return count;
}


struct blob *blob_get_blob_id(struct blob *bl, int id)
{
	struct blob *blp;

	for(blp = bl; blp != NULL && blp->id != id; blp = blp->next);

	return blp;
}


/*
void blob_set_rays(struct blob *bl, int rays)
{
	bl->rays = rays;
}


void blob_set_depth(struct blob *bl, int depth)
{
	bl->depth = depth;
}


void blob_set_rdtype(struct blob *bl, RD_TYPE type)
{
	bl->type = type;
}


void blob_set_bins(struct blob *bl, int bins)
{
	bl->bins = bins;
}


void blob_set_min(struct blob *bl, int min)
{
	bl->min = min;
}


void blob_set_max(struct blob *bl, int max)
{
	bl->max = max;
}


void blob_header_completed(struct blob *bl)
{
	bl->parse_header_flag = 1;
}
*/

#define BL_RD_ID     0x0001
#define BL_RD_RAYS   0x0002
#define BL_RD_TYPE   0x0004
#define BL_RD_BINS   0x0008
#define BL_RD_MIN    0x0010
#define BL_RD_MAX    0x0020
#define BL_RD_DEPTH  0x0040
#define BL_RD_COMPLET (BL_RD_ID|BL_RD_RAYS|BL_RD_TYPE|BL_RD_BINS|	\
		       BL_RD_MIN|BL_RD_MAX|BL_RD_DEPTH)



struct blob *blob_set_rawdata(struct blob *bl, const char **val)
{
	int i;
	int tag_sta = 0;
	struct blob *bl_id = NULL;

	for (i=0; val[i] && bl_id == NULL; i+=2) {
		if ( strncmp(val[i], "blobid", 6) == 0 ) {
			bl_id = blob_get_blob_id(bl, atoi(val[i+1]));
			tag_sta |= BL_RD_ID;
		}
	}
	for (i=0; val[i] && bl_id != NULL; i+=2) {
		if ( strncmp(val[i], "rays", 4) == 0 ) {
			bl_id->rays = atoi(val[i+1]);
			tag_sta |= BL_RD_RAYS;
		} else if ( strncmp(val[i], "type", 4) == 0 ) {
			if ( strncmp(val[i+1], "KDP", 3) == 0 )
				bl_id->type = KDP;
			else if ( strncmp(val[i+1], "dBZ", 3) == 0 )
				bl_id->type = dBZ;
			else if ( strncmp(val[i+1], "ZDR", 3) == 0 )
				bl_id->type = ZDR;
			else if ( strncmp(val[i+1], "PhiDP", 5) == 0 )
				bl_id->type = PhiDP;
			else 
				bl_id->type = UNKNOW;
			tag_sta |= BL_RD_TYPE;
		} else if ( strncmp(val[i], "bins", 4) == 0 ) {
			bl_id->bins = atoi(val[i+1]);
			tag_sta |= BL_RD_BINS;
		} else if ( strncmp(val[i], "min", 3) == 0 ) {
			bl_id->min = atof(val[i+1]);
			tag_sta |= BL_RD_MIN;
		} else if ( strncmp(val[i], "max", 3) == 0 ) {
			bl_id->max = atof(val[i+1]);
			tag_sta |= BL_RD_MAX;
		} else if ( strncmp(val[i], "depth", 5) == 0 ) {
			bl_id->depth = atoi(val[i+1]);
			tag_sta |= BL_RD_DEPTH;
		}
	}
	if ( tag_sta == BL_RD_COMPLET) {
		bl_id->parse_header_flag = 1; 
		BLOB_GET_BLOB_TYPE(bl_id) = BT_RAWDATA;
	} else {
		PVERB(PWARN,
		      "Incomplet header in xml (blobid=%d), RD sta: 0x%x.", 
		      BLOB_GET_ID(bl_id), tag_sta);
	}
	return bl_id;
}


/*
 <rayinfo refid="startangle" blobid="0" rays="361" depth="16" />
*/

#define BL_RI_ID     0x0001
#define BL_RI_RAYS   0x0002
#define BL_RI_DEPTH  0x0040
#define BL_RI_COMPLET (BL_RI_ID|BL_RI_RAYS|BL_RD_DEPTH)

struct blob *blob_set_rayinfo(struct blob *bl, const char **val)
{
	int i;
	int tag_sta = 0;
	struct blob *bl_id = NULL;

	for (i=0; val[i] && bl_id == NULL; i+=2) {
		if ( strncmp(val[i], "blobid", 6) == 0 ) {
			bl_id = blob_get_blob_id(bl, atoi(val[i+1]));
			tag_sta |= BL_RI_ID;
		}
	}
	for (i=0; val[i] && bl_id != NULL; i+=2) {
		if ( strncmp(val[i], "rays", 4) == 0 ) {
			bl_id->rays = atoi(val[i+1]);
			tag_sta |= BL_RI_RAYS;
		} else if ( strncmp(val[i], "depth", 5) == 0 ) {
			bl_id->depth = atoi(val[i+1]);
			tag_sta |= BL_RI_DEPTH;
		}
	}
	if ( tag_sta == BL_RI_COMPLET) {
		bl_id->parse_header_flag = 1; 
		BLOB_GET_BLOB_TYPE(bl_id) = BT_RAYINFO;
	} else {
		PVERB(PWARN,
		      "Incomplet header in xml (blobid=%d), RD sta: 0x%x.", 
		      BLOB_GET_ID(bl_id), tag_sta);
	}
	return bl_id;
}

