/* This is simple demonstration of how to use expat. This program
   reads an XML document from standard input and writes a line with
   the name of each element to standard output indenting child
   elements by one tab stop more than their parent element.
   It must be used with Expat compiled for UTF-8 output.
*/

#include <stdio.h>
#include "expat.h"

#if defined(__amigaos__) && defined(__USE_INLINE__)
#include <proto/expat.h>
#endif

#ifdef XML_LARGE_SIZE
#if defined(XML_USE_MSC_EXTENSIONS) && _MSC_VER < 1400
#define XML_FMT_INT_MOD "I64"
#else
#define XML_FMT_INT_MOD "ll"
#endif
#else
#define XML_FMT_INT_MOD "l"
#endif

static void XMLCALL
startElement(void *userData, const char *name, const char **atts)
{
  int i;
  int *depthPtr = (int *) XML_GetUserData(userData);
  for (i = 0; i < *depthPtr; i++)
    putchar('\t');
  puts(name);

  if ( atts[0] ){
	  for (i = 0; i <= *depthPtr; i++)
		  putchar('\t');
	  for (i = 0; atts[i]; i += 2) {
		  printf(" %s='%s',", atts[i], atts[i + 1]);
	  }
	  printf("\n");
  }

  *depthPtr += 1;
}

static void XMLCALL
endElement(void *userData, const char *name)
{
	int *depthPtr = (int *) XML_GetUserData(userData);

	*depthPtr -= 1;
}



int Eventcnt = 0;

static void XMLCALL
char_hndl(void *data, const char *txt, int txtlen) {
	//printf("%4d: Text - ", Eventcnt++);
  fwrite(txt, txtlen, sizeof(char), stdout);
}  /* End char_hndl */



static void XMLCALL
DefaultHandler(void *userData, const XML_Char *s, int len)
{
	printf("\nACA (%d): ",len);
	fwrite(s, len, sizeof(char), stdout);
}




int main(int argc, char *argv[])
{
	char buf[BUFSIZ];
	XML_Parser parser = XML_ParserCreate(NULL);
	int done;
	int depth = 0;
	XML_SetUserData(parser, &depth);
	XML_UseParserAsHandlerArg(parser);
	XML_SetElementHandler(parser, startElement, endElement);
	XML_SetCharacterDataHandler(parser, char_hndl);
	
	//XML_SetDefaultHandler(parser, DefaultHandler);

	do {
		int len = (int)fread(buf, 1, sizeof(buf), stdin);
		done = len < sizeof(buf);
		if (XML_Parse(parser, buf, len, done) == XML_STATUS_ERROR) {
			fprintf(stderr,
				"%s at line %" XML_FMT_INT_MOD "u\n",
				XML_ErrorString(XML_GetErrorCode(parser)),
				XML_GetCurrentLineNumber(parser));
			return 1;
		}
	} while (!done);


	XML_ParserFree(parser);
	return 0;
}
