/** 
 *   gevol: Gtk Extraer datos del VOLumen.
 *
 *   (c) Copyright 2010 Federico Bareilles <bareilles@gmail.com>,
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/


#define _GNU_SOURCE
#define GETTEXT_PACKAGE "gtk20"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <limits.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <glib.h>
#include <glib/gprintf.h>

#include <my_stdio.h>
#include "blob.h"

#define VOL_GET_PONT_CMD "vol_get_point"
#define LINE_BUFF_LEN 80

struct gevol_time {
	int year;
	int mon;
	int mday;
	int hour;
	int min;
	time_t t;
};

struct station {
	size_t blen;
	int out_of_range;
	char *buff;
	char *name;
	char *lat;
	char *lon;
	char *alt; /* not used */
	struct station *next;
}; 

#define POINT_FIX  0
#define POINT_LIST 1
#define POINT_ALL  2

int proc_point_mod = -1;

GtkWidget *window_main = NULL;
GtkWidget *filechooserbutton_in;
GtkWidget *filechooserbutton_out;
GtkWidget *filechooserbutton_point_list;
GtkWidget *combobox_grilla;
GtkWidget *entry_lat;
GtkWidget *entry_lon;
GtkWidget *entry_nodata;
GtkWidget *checkbutton_nodata;
GtkWidget *statusbar;
gint sb_context_id;

int year_from = -1;
struct gevol_time t_from;
struct gevol_time t_to;
int extraer[RD_TYPE_SIZE];
int grilla_id = 1; /* n*2 + 1 */


#define NODATA_STR (							\
		gtk_toggle_button_get_active (				\
			GTK_TOGGLE_BUTTON(checkbutton_nodata)) !=0 ?	\
		"exclude":(char *) gtk_entry_get_text(GTK_ENTRY(entry_nodata)))


static gint g_asprintf (gchar **string, gchar const *format, ...)
{
	va_list args;

	va_start(args, format);
	return g_vasprintf (&(*string), format, args);
}


static gchar* get_glade_filename (const gchar *name)
{
	gchar *filename = NULL;

	g_asprintf(&filename, PKGDATA_DIR "/%s.glade", name);

	return filename;
}


static struct station *station_make(void)
{
	struct station *st = (struct station *) malloc(sizeof(struct station));
	st->buff = malloc(sizeof(char) * LINE_BUFF_LEN);
	st->blen = LINE_BUFF_LEN;
	st->out_of_range = 0;
	st->name = NULL;
	st->lat = NULL;
	st->lon = NULL;
	st->alt = NULL;
	st->next = NULL;

	return st;
}


static void station_list_free(struct station *st)
{
	struct station *stp = st;

	while(stp != NULL) {
		st = stp->next;
		if (stp->buff != NULL) free(stp->buff);
		free(stp);
		stp = st;
	}
}


static void print_status_bar(const gchar *fmt, ...)
{
	va_list ap;
	gchar *buff;
	
	va_start(ap, fmt);
	g_vasprintf(&buff, fmt, ap);

	gtk_statusbar_pop (GTK_STATUSBAR (statusbar), sb_context_id);
	gtk_statusbar_push (GTK_STATUSBAR (statusbar),
			    sb_context_id, buff);

	while (gtk_events_pending ())
                gtk_main_iteration ();
	g_free(buff);

	return;
}


static int vol_proc(const char *dir,  /* directorio de entrada */
		    const char *name, /* archivo de entrada */
		    const char *dir_out,
		    const char *type, /* tipo de dato del archivo de entrada */
		    struct station *st, /* estacion terrena */
		    struct tm *from,  /* date from */
		    struct tm *to,    /* data to */
		    int overwrite_flag /* add or not add to file out */
	){
	char file_in[_POSIX_PATH_MAX];
	char file_out[_POSIX_PATH_MAX];
	char cmd[_POSIX_PATH_MAX];
	GError *error;
	gint exit_status;
	struct station *stp;

	if (st == NULL) { /* if st == NULL => extract all points. */
		char *basename = NULL;
		char *p;
		print_status_bar("Procesando: %s...", name);
		g_snprintf(file_in, _POSIX_PATH_MAX, "%s/%s", dir, name);
		g_asprintf(&basename, "%s", name);
		p = strchr(basename, '.');
		*p = 0;
		g_snprintf(file_out, _POSIX_PATH_MAX, "%s/%s.dat", 
			 dir_out, basename);
		free(basename);

		/* FIXME: Terminar de implementer la no salidad de
		 * valores sin dato...
		 */

		g_snprintf(cmd, _POSIX_PATH_MAX, VOL_GET_PONT_CMD		\
			 " --all -o %s --nodata %s  %s",
			 file_out, NODATA_STR, file_in);
		g_spawn_command_line_sync (cmd, NULL, NULL, 
					   &exit_status, &error);

		return exit_status;
	}
	for (stp = st; stp != NULL; stp = stp->next) {
		if (stp->out_of_range == 0) {
		g_snprintf(file_in, _POSIX_PATH_MAX, "%s/%s", dir, name);
		g_snprintf(file_out, _POSIX_PATH_MAX, 
			 "%s/%4d%02d%02d%02d%02d%4d%02d%02d%02d%02d%d%s_%s.dat", 
			 dir_out, 
			 from->tm_year+1900,from->tm_mon+1,from->tm_mday,
			 from->tm_hour,from->tm_min,
			 to->tm_year+1900,to->tm_mon+1,to->tm_mday,
			 to->tm_hour,to->tm_min,
			 (grilla_id << 1)|1, type, 
			 (stp->name != NULL)?stp->name:"fixpoint" );
		print_status_bar("Procesando: %s en %s, %s",
				 name, stp->lat, stp->lon);
		g_snprintf(cmd, _POSIX_PATH_MAX, VOL_GET_PONT_CMD		\
			 " --tags %s --lat %s --lon %s -o %s --griddim %d --nodata %s  %s",
			 (overwrite_flag == 0)?"--add":"",
			 stp->lat, stp->lon, file_out, 
			 (grilla_id << 1)|1, NODATA_STR, file_in);
		g_spawn_command_line_sync (cmd, NULL, NULL, 
					   &exit_status, &error);
		if (exit_status == 512) {
			unlink(file_out);
			stp->out_of_range = 1;
			if ( stp->name != NULL)
				print_status_bar("Estación %s fuera de rango", 
						 stp->name);	
			else
				print_status_bar("Punto fuera de rango");
		} 
		}
	}

	return exit_status;
}


static time_t gevol_time2tm(struct tm *tm, struct gevol_time *gt)
{
	struct tm local;

	if (tm == NULL) tm = &local;
	memset(tm, 0, sizeof(struct tm));
	tm->tm_year = gt->year - 1900;
	tm->tm_mon = gt->mon -1;
	tm->tm_mday = gt->mday;
	tm->tm_hour = gt->hour ;
	tm->tm_min = gt->min;
	tm->tm_sec = 0;
	tm->tm_isdst = -1; /* Ignorar los saltos de hora debido a los usos */
	gt->t = mktime(tm);
	localtime_r(&gt->t, tm);
	
	return gt->t;
}


static RD_TYPE str2type(const char *str)
{
	RD_TYPE type;

	if ( strncmp(str, "KDP", 3) == 0 )
		type = KDP;
	else if ( strncmp(str, "dBZ", 3) == 0 )
		type = dBZ;
	else if ( strncmp(str, "ZDR", 3) == 0 )
		type = ZDR;
	else if ( strncmp(str, "PhiDP", 5) == 0 )
		type = PhiDP;
	else
		type = UNKNOW;
	
	return type;
}


/*
           PROVINCIA DE BUENOS AIRES
NH0419.dat ARGERICH                         -38.60   -62.60    10  5
NH0196.DAT AZUL-AERO                        -36.75   -59.83   132  5
NH0221.DAT BAHIA BLANCA-AERO                -38.73   -62.17    83  5
NH0400.DAT BALCARCE-INTA                    -37.75   -58.30   130  5
NH0216.DAT BARROW-INTA                      -38.32   -60.25   120  5
NH0008.dat BENITO JUAREZ-AERO               -37.72   -59.78   207  5
NH0189.DAT BOLIVAR-AERO                     -36.25   -61.10    93  5
NH0343.DAT BORDENAVE-INTA                   -37.85   -63.02   212  5
NH1798.DAT BRAGADO-INTA                     -35.11   -60.49        6
01234567890123456789012345678901234567890123456789012345678901234567890
0         1         2         3         4         5         6         7

 */
static struct station *get_stations_list(char *file)
{
	FILE *fp;
	struct station *st = NULL;
	struct station *stp, *stlast = NULL;
	char *p;
	int count = 0;



	if ((fp = fopen(file, "r"))  == NULL) {
		GtkWidget *d;
		print_status_bar("No se puede abrir el archivo de estaciones");
		d = gtk_message_dialog_new (GTK_WINDOW (window_main),
					    GTK_DIALOG_DESTROY_WITH_PARENT,
					    GTK_MESSAGE_ERROR,
					    GTK_BUTTONS_CLOSE,
					    "No se puede abrir el archivo de estaciones '%s'",
					    file);
		gtk_dialog_run (GTK_DIALOG (d));
		gtk_widget_destroy (d);
		return st;
	}

	st = station_make();
	stp = st;
	print_status_bar("Leyendo estaciones desde: %s", file);
	while (getline(&stp->buff, &stp->blen, fp) > 0) {
		if ( *(stp->buff) == 'N') {
			count++;
			*(stp->buff+10) = *(stp->buff+42) = 
				*(stp->buff+50) = *(stp->buff+59) = 
				*(stp->buff+65) = 0;
			stp->name = stripwhite(stp->buff+11);
			stp->lat = stripwhite(stp->buff+43);
			stp->lon = stripwhite(stp->buff+51);
			stp->alt = stripwhite(stp->buff+60);
			while((p = strchr(stp->name, ' ')) != NULL)
				*p = '_';
			while((p = strchr(stp->name, '/')) != NULL)
				*p = '-';
			while((p = strchr(stp->name, '<')) != NULL)
				*p = '-';
			while((p = strchr(stp->name, '>')) != NULL)
				*p = '-';
			stlast = stp;
			stp->next = station_make();
			stp = stp->next;
		}
	}
	if(stlast != NULL) {
		station_list_free(stp);
		stlast->next = NULL;
	}
	fclose(fp);
	
	if ( count == 0 ) {
		GtkWidget *d;
		station_list_free(st);
		st = NULL;
		d = gtk_message_dialog_new (GTK_WINDOW (window_main),
					    GTK_DIALOG_DESTROY_WITH_PARENT,
					    GTK_MESSAGE_ERROR,
					    GTK_BUTTONS_CLOSE,
					    "No se encontraron estaciones en: %s",					    file);
		gtk_dialog_run (GTK_DIALOG (d));
		gtk_widget_destroy (d);
	}

	return st;
}




void on_button_extraer_clicked( GtkWidget *widget, gpointer data)
{
	GError *error;
	GPatternSpec *filepattern = 
		g_pattern_spec_new("????????????????*.vol"); 
	struct tm tm_from, tm_to;
	char *dir_in = gtk_file_chooser_get_filename(
		GTK_FILE_CHOOSER(filechooserbutton_in));
	GDir *gdir_in = NULL;
	char *dir_out = gtk_file_chooser_get_filename(
		GTK_FILE_CHOOSER(filechooserbutton_out));
	char *file_poin_list = gtk_file_chooser_get_filename(
		GTK_FILE_CHOOSER(filechooserbutton_point_list));
	const gchar *name;
	struct gevol_time file_time;
	char type_str[80];
	int type_init[RD_TYPE_SIZE];
	RD_TYPE type;
	struct station st_fix = {
		.out_of_range = 0,
		.name = NULL,
		.lat = (char *) gtk_entry_get_text(GTK_ENTRY(entry_lat)),
		.lon = (char *) gtk_entry_get_text(GTK_ENTRY(entry_lon)),
		.next = NULL,
	};
	struct station *st = NULL;
	int count = 0;

	if (GTK_WIDGET_IS_SENSITIVE(combobox_grilla)) {
		if (GTK_WIDGET_IS_SENSITIVE(filechooserbutton_point_list) ){
			st = get_stations_list(file_poin_list);
			if (st == NULL)
				return;
			proc_point_mod = POINT_LIST;
		} else {
			st = &st_fix;
			proc_point_mod = POINT_FIX;
		}
	} else { /* if st == NULL => extract all points. */
		proc_point_mod = POINT_ALL;
	}
	gtk_widget_set_sensitive(GTK_WIDGET(data), 0);
	while (gtk_events_pending ()) gtk_main_iteration ();

	gevol_time2tm(&tm_from, &t_from);
	gevol_time2tm(&tm_to, &t_to);
	memset(type_init, 0, RD_TYPE_SIZE);
	gdir_in = g_dir_open(dir_in, 0, &error);
	while((name = g_dir_read_name(gdir_in))) {
		if(g_pattern_match_string (filepattern, name)) {
			//g_printf("File: %s\n", name);
			sscanf(name, "%4d%2d%2d%2d%2d", 
			       &file_time.year,
			       &file_time.mon, &file_time.mday,
			       &file_time.hour, &file_time.min);
			sscanf(name+16, "%[A-Za-z]s.vol", type_str);
			gevol_time2tm(NULL, &file_time);
			type = str2type(type_str);
			if ( (file_time.t >= t_from.t) &&
			     (file_time.t <= t_to.t)   &&
			     extraer[type]) {
				vol_proc(dir_in, name, dir_out, type_str, st, 
					 &tm_from, &tm_to,
					 (type_init[type] == 0)?1:0
					);
				type_init[type] = 1;
				count++;
			}
		}
	}
	if ( count == 0 ) {
		GtkWidget *d;
		station_list_free(st);
		st = NULL;
		d = gtk_message_dialog_new (GTK_WINDOW (window_main),
					    GTK_DIALOG_DESTROY_WITH_PARENT,
					    GTK_MESSAGE_ERROR,
					    GTK_BUTTONS_CLOSE,
					    "No se encontraron archivos de volumenes en: '%s', para el rango de fechas.",	
					    dir_in				    );
		gtk_dialog_run (GTK_DIALOG (d));
		gtk_widget_destroy (d);
	}
	print_status_bar("Terminado.");
#if 0
	if (GTK_WIDGET_IS_SENSITIVE(combobox_grilla)) 
		if (GTK_WIDGET_IS_SENSITIVE(filechooserbutton_point_list) )
			g_printf("Points list: %s\n", file_poin_list);
		else
			g_printf("Points fix: (%s, %s)\n", 
				 gtk_entry_get_text(GTK_ENTRY(entry_lat)),
				 gtk_entry_get_text(GTK_ENTRY(entry_lon)));
	else
		g_printf("All points\n");
#endif
#if 0
	g_printf("Procesar %d\n",t_to.hour);
	g_printf("Dir in: %s\n", dir_in);
	g_printf("Dir out: %s\n", dir_out);
	g_printf("From: %s", asctime(&tm_from));
	g_printf("To  : %s", asctime(&tm_to));
	g_printf("Extraer: KDP: %d, dBZ: %d, ZDR: %d, PhiDP: %d\n",
		 extraer[KDP],extraer[dBZ],extraer[ZDR],extraer[PhiDP]);
	g_printf("Grilla %dx%d\n", (grilla_id << 1)|1, (grilla_id << 1)|1);

#endif
	if (proc_point_mod == POINT_LIST ){
		station_list_free(st);
	}
	g_dir_close (gdir_in);

	gtk_widget_set_sensitive(GTK_WIDGET(data), 1);

	return;
}


#if 0
void on_aboutdialog_close( GtkWidget *widget, gpointer data)
{
	g_printf("ACA\n");
	gtk_widget_hide (data);	
}
#endif


void  spinbutton_time_change (GtkSpinButton *spinbutton,
			      gpointer       user_data)
{
	*((int *) user_data) = gtk_spin_button_get_value_as_int (spinbutton);
	
	return;
}


GtkWidget *spinbutton_connect_to (GladeXML *xml, char *name, gpointer data)
{
	GtkWidget *spinbutton = glade_xml_get_widget(xml, name);

	*((int *) data)= gtk_spin_button_get_value_as_int (
		GTK_SPIN_BUTTON(spinbutton));
	g_signal_connect_after(
		G_OBJECT (spinbutton), "value-changed",
		G_CALLBACK (spinbutton_time_change),  
		data);

	return spinbutton;
}


void  checkbutton_change (GtkToggleButton *checkbutton,
			  gpointer       user_data)
{
	*((int *) user_data) = gtk_toggle_button_get_active (checkbutton);
	
	return;
}


GtkWidget *checkbutton_connect_to (GladeXML *xml, char *name, gpointer data)
{
	GtkWidget *checkbutton = glade_xml_get_widget(xml, name);

	*((int *) data)= gtk_toggle_button_get_active (
		GTK_TOGGLE_BUTTON(checkbutton));
	g_signal_connect_after(
		G_OBJECT (checkbutton), "toggled",
		G_CALLBACK (checkbutton_change),  
		data);

	return checkbutton;
}


void  combobox_change (GtkComboBox *button,
			  gpointer       user_data)
{
	*((int *) user_data) = gtk_combo_box_get_active (button);
	
	return;
}


void  radiobutton_toggled (GtkComboBox *button,
			  gpointer data)
{
	gtk_widget_set_sensitive(GTK_WIDGET(data),
				 gtk_toggle_button_get_active(
					 GTK_TOGGLE_BUTTON(button)));
	return;
}


void  radiobutton_toggled_inv (GtkComboBox *button,
			  gpointer data)
{
	gtk_widget_set_sensitive(GTK_WIDGET(data),
				 !gtk_toggle_button_get_active(
					 GTK_TOGGLE_BUTTON(button)));
	return;
}



int main(int argc, char *argv[]) {
        GladeXML *xml;
	char *glade_filename = NULL;
	GtkWidget *radiobutton_point_list;
	GtkWidget *radiobutton_point_fix;
	GtkWidget *radiobutton_point_all;
	GtkWidget *eventbox_point_fix;
	GtkWidget *button_extraer;
	GtkWidget *vbox_main;
	GtkFileFilter *est_filter;
	
	gtk_init(&argc, &argv);
	glade_filename = get_glade_filename("gevol");
        /* load the interface */
	xml = glade_xml_new (glade_filename, NULL, NULL);
	if (!xml) {
		PVERB (PERR, "Could not load \"%s\"", glade_filename);
		free(glade_filename);
		exit (1);
	}
	glade_xml_signal_autoconnect(xml);
	window_main = glade_xml_get_widget(xml, "window_main");
	filechooserbutton_in = 
		glade_xml_get_widget(xml, "filechooserbutton_in");
	filechooserbutton_out = 
		glade_xml_get_widget(xml, "filechooserbutton_out");
	filechooserbutton_point_list = 
		glade_xml_get_widget(xml, "filechooserbutton_point_list");

	est_filter = gtk_file_filter_new(); 
	gtk_file_filter_set_name(est_filter, "Archivos dat");
	gtk_file_filter_add_pattern(est_filter, "*.[Dd][Aa][Tat]");
	gtk_file_chooser_add_filter( 
		GTK_FILE_CHOOSER(filechooserbutton_point_list), est_filter);

	est_filter = gtk_file_filter_new(); 
	gtk_file_filter_set_name(est_filter, "Todos");
	gtk_file_filter_add_pattern(est_filter, "*");
	gtk_file_chooser_add_filter( 
		GTK_FILE_CHOOSER(filechooserbutton_point_list), est_filter);
	
	combobox_grilla = glade_xml_get_widget(xml, "combobox_grilla");
	gtk_combo_box_set_active(GTK_COMBO_BOX(combobox_grilla), grilla_id);
	g_signal_connect_after( G_OBJECT (combobox_grilla), "changed",
				G_CALLBACK (combobox_change),  &grilla_id);

	spinbutton_connect_to(xml, "spinbutton_year_from", &t_from.year);
	spinbutton_connect_to(xml, "spinbutton_mon_from",  &t_from.mon);
	spinbutton_connect_to(xml, "spinbutton_day_from",  &t_from.mday);
	spinbutton_connect_to(xml, "spinbutton_hour_from", &t_from.hour);
	spinbutton_connect_to(xml, "spinbutton_min_from",  &t_from.min);

	spinbutton_connect_to(xml, "spinbutton_year_to", &t_to.year);
	spinbutton_connect_to(xml, "spinbutton_mon_to",  &t_to.mon);
	spinbutton_connect_to(xml, "spinbutton_day_to",  &t_to.mday);
	spinbutton_connect_to(xml, "spinbutton_hour_to", &t_to.hour);
	spinbutton_connect_to(xml, "spinbutton_min_to",  &t_to.min);

	checkbutton_connect_to(xml, "checkbutton_kdp",   &extraer[KDP]);
	checkbutton_connect_to(xml, "checkbutton_dbz",   &extraer[dBZ]);
	checkbutton_connect_to(xml, "checkbutton_zdr",   &extraer[ZDR]);
	checkbutton_connect_to(xml, "checkbutton_phidp", &extraer[PhiDP]);

	radiobutton_point_list = 
		glade_xml_get_widget(xml, "radiobutton_point_list");
	radiobutton_point_fix = 
		glade_xml_get_widget(xml, "radiobutton_point_fix");
	eventbox_point_fix = 
		glade_xml_get_widget(xml, "eventbox_point_fix");
	radiobutton_point_all = 
		glade_xml_get_widget(xml, "radiobutton_point_all");

	g_signal_connect_after( G_OBJECT (radiobutton_point_list),
				"toggled", 
				G_CALLBACK (radiobutton_toggled),
				filechooserbutton_point_list);
	g_signal_connect_after( G_OBJECT (radiobutton_point_fix),
				"toggled", 
				G_CALLBACK (radiobutton_toggled),
				eventbox_point_fix);
	g_signal_connect_after( G_OBJECT (radiobutton_point_all),
				"toggled", 
				G_CALLBACK (radiobutton_toggled_inv),
				combobox_grilla);

	entry_lat = glade_xml_get_widget(xml, "entry_lat");
	entry_lon = glade_xml_get_widget(xml, "entry_lon");
	entry_nodata = glade_xml_get_widget(xml, "entry_nodata");
	checkbutton_nodata = glade_xml_get_widget(xml, "checkbutton_nodata");

	g_signal_connect_after( G_OBJECT (checkbutton_nodata),
				"toggled", 
				G_CALLBACK (radiobutton_toggled_inv),
				entry_nodata);

	statusbar = glade_xml_get_widget(xml, "statusbar");
	sb_context_id = gtk_statusbar_get_context_id(
                GTK_STATUSBAR (statusbar), "...");

	button_extraer = glade_xml_get_widget(xml, "button_extraer");
	vbox_main = glade_xml_get_widget(xml, "vbox_main");
	g_signal_connect_after( G_OBJECT (button_extraer),
				"clicked", 
				G_CALLBACK (on_button_extraer_clicked),
				vbox_main);

        gtk_widget_show (window_main);
	gtk_main();

	free(glade_filename);

        return 0;
}


