/** 
 *   kdp2dbz: Transformar KDP a dBZ segun el algoritmo de
 *   kdp2dbz_func.c.
 *
 *   (c) Copyright 2009 Federico Bareilles <bareilles@gmail.com>,
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/
#define _GNU_SOURCE

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdint.h>
#include <getopt.h>
#include <fcntl.h>
#include <my_stdio.h>

#include "version.h"
#include "my_xcomp.h"
#include "blob.h"
#include "xml_proc.h"
#include "kdp2dbz_func.h"

int compress_flag = 1;


const char *name_definition = {"KDP to dBZ converter"};
static const char *help_text[] ={
#       ifdef VERBOSE_MODE
        "  -v\t\t - verbose mode (incremental).",
#       endif
	"  --uncompress\t - No compress binary blocks",
        "  -V\t\t - Show program version.",
	"  -o\t\t - File out, default stdout.",
	"  -a\t\t - Paramerte a.",
	"  -b\t\t - Paramerte b.",
	"  -c\t\t - Paramerte c.",
	"  --noautoescale - No auto escale data out.",
        "  -h [--help]\t - this help !!!.",
        ""
};

void version(char *program_name)
{
        fprintf(stderr,"%s: ", name_definition);
        fprintf(stderr,"%s version %s\n", program_name, VERSION);
}


void usage(char *program_name)
{
        version(program_name);
        fprintf(stderr,
#               ifdef VERBOSE_MODE
 "Usage: \t%s [-vhV][--uncompress]][--help][-o file_out][-a par_a][-b par_b][-c par_c] (src_file_name|-)\n",
#                else
 "Usage: \t%s [-hV][--uncompress]][--help][-o file_out][-a par_a][-b par_b][-c par_c] (src_file_name|-)\n",
#               endif
                program_name);
}


void help()
{
        int i=0;

        while((char) *help_text[i] != 0)
                fprintf(stderr,"%s\n",help_text[i++]);
        fprintf(stderr,"\n");
}


int main(int argc, char **argv)
{
	struct blob *bl = NULL;
	XML_Parser parser = xmlp_init();
        char *program_name;
	int blob_count;
	char *file_in = NULL;
	char *file_out = NULL;
        char *cp;
	int op;
        struct option const longopts[] =
        {
                {"help", 0, 0, 'h'},
                {"verbose", 0, 0, 'v'},
		{"uncompress", 0, 0, 201},
		{"noautoescale", 0, 0, 202},
                {0,  0,  0,  0}
        };
	struct param_KDP2dBZ k2d;

	k2d.a = 4.0;
	k2d.b = 0.93;
	k2d.c = 0.0;

	k2d.min = DBZ_MIN;
	k2d.max = DBZ_MAX;
	k2d.autoescale = 1;


        /* who is my name? */
        if ((cp = strrchr(argv[0], '/')) != NULL) program_name  = cp + 1;
        else program_name  = argv[0];

        verbose_flag = PERR;
        while ((op = getopt_long(argc, argv, "vhVo:a:b:c:",longopts, 0)) != EOF)
                switch( op ){
		case 'a':
			k2d.a = atof(optarg);
			break;
		case 'b':
			k2d.b = atof(optarg);
			break;
		case 'c':
			k2d.c = atof(optarg);
			break;
		case 'o':
			file_out = optarg;
			if (strlen(optarg) == 1 && *(optarg) == '-')
				file_out = NULL;
			break;
		case 201:
			compress_flag = 0;
			break;
		case 202:
			k2d.autoescale = 0;
			break;
                case 'v':
                        verbose_flag = (verbose_flag << 1) | 1;
                        break;
                case 'V':
                        version(program_name);
                        exit(1);
                        break;
                case 'h':
			usage(program_name);
                        help();
                        exit(1);
                default:
                        usage(program_name);
                        exit(1);
                }
	
	file_in = argv[optind];

	if ((blob_count = blob_get_file(file_in, &bl)) < 1 ) {
		PVERB(PERR,"Bad input file: %s", file_in);
		return 1;
	}

	XMLP_SET_BLOB(parser, bl);

	//XMLP_SET_PROC_PARAMETERS(parser, NULL);
	XMLP_SET_PROC_PARAMETERS(parser, &k2d);
	//XMLP_SET_PROC_RAWDATA(parser, NULL);
	XMLP_SET_PROC_RAWDATA(parser, proc_rawdata_0);

	if (file_out == NULL)
		XMLP_SET_FPO(parser, stdout);
	else
		xmlp_open_file_out(parser, file_out);
	xmlp_run_on(parser, file_in);
	fprintf(XMLP_GET_FPO(parser),"<!-- END XML -->\n");
	blob_print(XMLP_GET_FPO(parser), bl, compress_flag);
	if (file_out != NULL)
		xmlp_close_file_out(parser);
	xmlp_free(parser);
	free_blob_all(bl);
	
	return 0;
}
	
		
	
