/** 
 *   kdp2dbz_func: Transformacion de los datos en KDP a dBZ
 *
 *   (c) Copyright 2010 Federico Bareilles <bareilles@gmail.com>,
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *   
 **/

#define _GNU_SOURCE

#include <my_stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <math.h>
//#include <complex.h>

#include "blob.h"
#include "my_xcomp.h"
#include "kdp2dbz_func.h"

/*
#define KDP_UINT2DOUBLE(bl, i)				  \
	(double) (BLOB_GET_DATA_UINT16(bl, i) - 1) *	  \
	(BLOB_GET_MAX(bl) - BLOB_GET_MIN(bl)) / 65534.0 + \
	BLOB_GET_MIN(bl)
*/
#define KDP_UINT2DOUBLE(bl, i) BLOB_GET_VALUE_SCALE(bl, i)

#define DBZ_DOUBLE2UINT8(val, min, max)			\
  (uint8_t) ( (val - min) * 254.0 / (max - min) + 1.0)
/*
#define DBZ_UINT82DOUBLE(val, min, max)		\
  (((double) val - 1.0) * (max - min) / 254.0) + min 
*/
/*
#define DBZ_UINT82DOUBLE(bl, i)				  \
	(double) (BLOB_GET_DATA_UINT8(bl, i) - 1) *	  \
	(BLOB_GET_MAX(bl) - BLOB_GET_MIN(bl)) / 254.0 +	  \
	BLOB_GET_MIN(bl)
*/



static int if_valid_kdp_data(struct blob *bl)
{
	if ( BLOB_GET_TYPE(bl) != KDP ) {
		PVERB(PWARN, "BLOB id: %d, is no type KDP.", BLOB_GET_ID(bl));
		return 0;
	}
	if ( BLOB_GET_DEPTH(bl) != 16 ) {
		PVERB(PWARN, "BLOB id: %d, invalid depth: %d.", 
		      BLOB_GET_ID(bl), BLOB_GET_DEPTH(bl));
		return 0;
	}

	return 1;
}


static double kdp2dbz_0(int *data_valid, double kdp, struct param_KDP2dBZ *p)
{
	double dbz;
	//double complex lkdp = clog10((double complex) kdp);

	/* TODO: ecuacion de transformacion.... */
	if( kdp <= 0.0) {
		*data_valid = 0;
		return 0.0;
	}
	*data_valid = 1;
	//dbz = pow(10.0,(p->a + log10(kdp)) / p->b);
	dbz = (p->a + log10(kdp)) / p->b;

	return dbz;
}


int proc_rawdata_0(struct blob *bl, void *parameters)
{
	struct param_KDP2dBZ *p = (struct param_KDP2dBZ *) parameters;
	uint16_t *data;
	uint8_t *data_out;
	int data_valid;
	double *dbz;
	int i;
	int size = BLOB_GET_RAYS(bl) * BLOB_GET_BINS(bl);

	if (!if_valid_kdp_data(bl) )
		return -1;

	data_out = (uint8_t *) malloc(sizeof(uint8_t) * size);
	dbz = (double *) malloc(sizeof(double) * size);
	blob_uncompress(bl);
	data = (uint16_t *) BLOB_GET_DATA(bl);

	if( p->autoescale) {
		p->min = DBZ_MAX;
		p->max = DBZ_MIN;
	}
	for(i=0; i < size; i++) {
		if ( *(data+i) != 0 ) {
			*(dbz+i) = kdp2dbz_0(&data_valid, 
					     KDP_UINT2DOUBLE(bl, i), p);
			if (data_valid)	{
				*(data_out+i) = 1;
				if (p->autoescale) {
					if (*(dbz+i) > p->max) p->max = *(dbz+i);
					if (*(dbz+i) < p->min) p->min = *(dbz+i);
				}
			} else *(data_out+i) = 0;
		} else {
			*(data_out+i) = 0;
		}
	}
	if( p->autoescale) {
		p->min = floor(p->min * 100.0) / 100.0;
		p->max = ceil(p->max * 100.0) / 100.0;
		if ( p->min < DBZ_MIN) p->min = DBZ_MIN;
		if ( p->max > DBZ_MAX) p->min = DBZ_MAX;
	}
	for(i=0; i < size; i++) {
		if ( *(data_out+i) != 0 ) {
			*(data_out+i) =	DBZ_DOUBLE2UINT8(*(dbz+i), 
							 p->min, p->max);
			//if (BLOB_GET_ID(bl) == 1)
			//printf("%3d %f\n", *(data_out+i), *(dbz+i));
		}
		/*
		if (BLOB_GET_ID(bl) == 1)
			printf("%3d  %f\n", *(data_out+i),  *(dbz+i));
		*/

	}
	free(dbz);
	free(BLOB_GET_DATA(bl));
	BLOB_SET_DATA(bl, data_out);
	BLOB_SET_MIN(bl, p->min);
	BLOB_SET_MAX(bl, p->max);
	BLOB_SET_TYPE(bl, dBZ);
	BLOB_SET_DEPTH(bl, 8);
	BLOB_SET_DATA_SIZE(bl, sizeof(uint8_t) * size);

	return 0;
}
